package wtf.hak.survivalfabric.mixin.client;

import net.minecraft.block.enums.CameraSubmersionType;
import net.minecraft.client.render.BackgroundRenderer;
import net.minecraft.client.render.Camera;
import net.minecraft.client.render.Fog;
import net.minecraft.client.render.FogShape;
import net.minecraft.world.World;
import org.joml.Vector4f;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

import static wtf.hak.survivalfabric.config.client.ClientConfigManager.getConfig;

@Mixin(value = BackgroundRenderer.class, priority = 910)
public class BackgroundRendererMixin {

    @Unique
    private static final Fog EMPTY_FOG = new Fog(-8.0f, 1_000_000.0F, FogShape.CYLINDER, 0, 0, 0, 0);

    @Inject(method = "applyFog", at = @At("RETURN"), cancellable = true)
    private static void applyFog(Camera camera, BackgroundRenderer.FogType fogType, Vector4f color, float viewDistance, boolean thickenFog, float tickProgress, CallbackInfoReturnable<Fog> cir) {
        CameraSubmersionType submersion = camera.getSubmersionType();
        boolean renderFog = true;

        switch (submersion) {
            case NONE -> {
                World world = camera.getFocusedEntity().getWorld();
                if ((world.getRegistryKey() == World.OVERWORLD && !getConfig().renderOverworldFog)
                        || (world.getRegistryKey() == World.NETHER && !getConfig().renderNetherFog)
                        || (world.getRegistryKey() == World.END && !getConfig().renderEndFog)) {
                    renderFog = false;
                }
            }
            case WATER -> renderFog = getConfig().renderWaterFog;
            case LAVA -> renderFog = getConfig().renderLavaFog;
            case POWDER_SNOW -> renderFog = getConfig().renderSnowFog;
        }

        if (!renderFog) {
            cir.setReturnValue(EMPTY_FOG);
        }
    }
}
